;--------------------------------
;Include Modern UI
!include "MUI2.nsh"

!include "x64.nsh" ;detect architecture (32/64 bit)

; Section define/macro header file
!include "Sections.nsh"

!include "FileFunc.nsh"
!insertmacro RefreshShellIcons
!insertmacro un.RefreshShellIcons


;--------------------------------
;General

  ;Name and file
  !include "..\..\version\version.nsh"

  ;Default installation folder
  InstallDir "$PROGRAMFILES64\FreeFileSync" ;$PROGRAMFILES expands to "Program files (x86)" on 64-bit Windows, we don't want this
  
  ;Get installation folder from registry (only if available)
  InstallDirRegKey HKLM "Software\FreeFileSync" "InstallDir"

  ;Request application privileges for Windows Vista
  RequestExecutionLevel admin

;--------------------------------
;Interface Configuration
  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_BITMAP "header.bmp"
  !define MUI_ICON "install.ico"
  !define MUI_UNICON "uninstall.ico"

;--------------------------------
;Pages

  !insertmacro MUI_PAGE_LICENSE "..\..\BUILD\License.txt"
  Page custom nsDialogsPage nsDialogsPageLeave
  ;!insertmacro MUI_PAGE_COMPONENTS
  ;!insertmacro MUI_PAGE_DIRECTORY
  !insertmacro MUI_PAGE_INSTFILES
  
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  
;--------------------------------
;Languages
  !include "Lang\english.nsh" ;first language is the default language
  !include "Lang\arabic.nsh" 
  !include "Lang\danish.nsh" 
  !include "Lang\hebrew.nsh" 
  !include "Lang\german.nsh"
  !include "Lang\french.nsh"
  !include "Lang\greek.nsh" 
  !include "Lang\korean.nsh" 
  !include "Lang\russian.nsh"
  !include "Lang\swedish.nsh" 
  !include "Lang\romanian.nsh" 
  !include "Lang\chinese_simple.nsh"
  !include "Lang\chinese_traditional.nsh"
  !include "Lang\polish.nsh"
  !include "Lang\hungarian.nsh"
  !include "Lang\portuguese.nsh"
  !include "Lang\portuguese_br.nsh"
  !include "Lang\czech.nsh"
  !include "Lang\dutch.nsh"
  !include "Lang\slovenian.nsh"
  !include "Lang\spanish.nsh"
  !include "Lang\italian.nsh"
  !include "Lang\japanese.nsh"
  !include "Lang\finnish.nsh"  
  !include "Lang\ukrainian.nsh" 
;--------------------------------

;##################  PROGRAM FILES  ################################################################################

Section "main files" 
  SetOutPath "$INSTDIR"
  File "..\..\BUILD\Resources.zip"
  File "..\..\BUILD\Changelog.txt"
  File "..\..\BUILD\License.txt"
  File "..\..\BUILD\FreeFileSync.chm"
  File "..\..\BUILD\Invisible.vbs"
  File "..\..\BUILD\Sync_Complete.wav"
  File "..\..\BUILD\Compare_Complete.wav"
  File "..\..\BUILD\FreeFileSync.exe"
  File "..\..\BUILD\RealtimeSync.exe"
  
  SetOutPath "$INSTDIR\Languages"
  File "..\..\BUILD\Languages\*.lng"
  SetOutPath "$INSTDIR"
  
  SetOutPath "$INSTDIR\Bin"
  ;32-Bit files
  File "..\..\BUILD\Bin\FreeFileSync_Win32.exe"
  File "..\..\BUILD\Bin\RealtimeSync_Win32.exe"
  File "..\..\BUILD\Bin\mingwm10.dll" ;MINGW currently needs this one
  File "..\..\BUILD\Bin\Shadow_Server2003_Win32.dll"
  File "..\..\BUILD\Bin\Shadow_XP_Win32.dll"
  File "..\..\BUILD\Bin\FileOperation_Win32.dll"
  File "..\..\BUILD\Bin\Taskbar7_Win32.dll"  
  File "..\..\BUILD\Bin\Thumbnail_Win32.dll"  
  
  ;64-Bit files
  File "..\..\BUILD\Bin\FreeFileSync_x64.exe"
  File "..\..\BUILD\Bin\RealtimeSync_x64.exe"
  File "..\..\BUILD\Bin\Shadow_Server2003_x64.dll"
  File "..\..\BUILD\Bin\Shadow_XP_x64.dll"
  File "..\..\BUILD\Bin\FileOperation_x64.dll" 
  File "..\..\BUILD\Bin\Taskbar7_x64.dll" 
  File "..\..\BUILD\Bin\Thumbnail_x64.dll"  
  SetOutPath "$INSTDIR"
SectionEnd
;###############################################################################################################

;handles for user selections
Var InstallTypeLocal
Var InstallTypeDescription
;Var Architecture32Bit
Var CheckBoxDesktopLink
Var CheckBoxStartmenuLink
Var SelectedDirectory


Section "Install type: local" SecInstallTypeInstaller
  ;create uninstaller
  WriteUninstaller "$INSTDIR\uninstall.exe"

  ;Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\FreeFileSync" "DisplayName" "${FFS_Version_Str}"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\FreeFileSync" "DisplayVersion" "${FFS_Version_Id}"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\FreeFileSync" "DisplayIcon" '"$INSTDIR\FreeFileSync.exe"'
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\FreeFileSync" "Publisher" "ZenJu"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\FreeFileSync" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\FreeFileSync" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\FreeFileSync" "NoRepair" 1

  ;print info about install progress
  DetailPrint $(Lang_Registering_FreeFileSync_file_extensions)
  
  ;register *.ffs_gui files
  WriteRegStr HKLM "Software\Classes\.ffs_gui\shell\open\command" "" '"$INSTDIR\FreeFileSync.exe" "%1"'
  WriteRegStr HKLM "Software\Classes\.ffs_gui\DefaultIcon" "" '$INSTDIR\FreeFileSync.exe,0'
  
  ;register *.ffs_batch files
  WriteRegStr HKLM "Software\Classes\.ffs_batch\shell\open\command" "" '"$INSTDIR\FreeFileSync.exe" "%1"'
  WriteRegStr HKLM "Software\Classes\.ffs_batch\DefaultIcon" "" '$INSTDIR\FreeFileSync.exe,1'

  ;register *.ffs_db files
  WriteRegStr HKLM "Software\Classes\.ffs_db\DefaultIcon" "" '$INSTDIR\FreeFileSync.exe,2'

  ;register *.ffs_real files
  WriteRegStr HKLM "Software\Classes\.ffs_real\shell\open\command" "" '"$INSTDIR\RealtimeSync.exe" "%1"'
  WriteRegStr HKLM "Software\Classes\.ffs_real\DefaultIcon" "" '$INSTDIR\RealtimeSync.exe,0'
  ${RefreshShellIcons}
  
  ;remember installation directory (still no registry access for "portable" installation type!)
  WriteRegStr HKLM "Software\FreeFileSync" "InstallDir" "$INSTDIR"
SectionEnd


Section "Desktop" SecDesktopShortcut
	SetShellVarContext all ;create shortcuts for all users
	CreateShortCut "$DESKTOP\FreeFileSync.lnk" "$INSTDIR\FreeFileSync.exe"
	SetShellVarContext current ;restore value
SectionEnd


Section "Startmenu" SecStartmenuEntry
	SetShellVarContext all ;create shortcuts for all users
	CreateDirectory "$SMPROGRAMS\FreeFileSync"
	CreateShortCut "$SMPROGRAMS\FreeFileSync\FreeFileSync.lnk" "$INSTDIR\FreeFileSync.exe"
	CreateShortCut "$SMPROGRAMS\FreeFileSync\RealtimeSync.lnk" "$INSTDIR\RealtimeSync.exe"
	CreateShortCut "$SMPROGRAMS\FreeFileSync\$(Lang_Help).lnk" "$INSTDIR\FreeFileSync.chm"
	SetShellVarContext current ;restore value

	;write uninstaller link in startmenu if appropriate only
	SectionGetFlags ${SecInstallTypeInstaller} $0
	IntOp $0 $0 & ${SF_SELECTED}
	${If} $0 != 0
		SetShellVarContext all ;create shortcuts for all users
		CreateShortCut "$SMPROGRAMS\FreeFileSync\$(Lang_Uninstall).lnk" "$INSTDIR\uninstall.exe"
		SetShellVarContext current ;restore value
	${EndIf}
SectionEnd


Section "Uninstall"
	;remove files in user application directory
  Delete "$APPDATA\FreeFileSync\LastRun.ffs_gui"
  Delete "$APPDATA\FreeFileSync\LastRun.ffs_real"
  Delete "$APPDATA\FreeFileSync\GlobalSettings.xml"
  Delete "$APPDATA\FreeFileSync\LastError.txt"
  Delete "$APPDATA\FreeFileSync\Logs\*.log"
  RMDir "$APPDATA\FreeFileSync\Logs"
  RMDir "$APPDATA\FreeFileSync" ;remove if empty

  ;remove files in FreeFileSync installation directory
  Delete "$INSTDIR\Resources.zip"
  Delete "$INSTDIR\Resources.dat"
  Delete "$INSTDIR\Changelog.txt"
  Delete "$INSTDIR\FreeFileSync.exe"
  Delete "$INSTDIR\RealtimeSync.exe"
  Delete "$INSTDIR\License.txt"
  Delete "$INSTDIR\Invisible.vbs"
  Delete "$INSTDIR\FreeFileSync.chm"  
  Delete "$INSTDIR\Sync_Complete.wav"
  Delete "$INSTDIR\Compare_Complete.wav" 
  Delete "$INSTDIR\uninstall.exe"
  
  Delete "$INSTDIR\Bin\mingwm10.dll"
  Delete "$INSTDIR\Bin\FreeFileSync_Win32.exe"
  Delete "$INSTDIR\Bin\FreeFileSync_x64.exe"
  Delete "$INSTDIR\Bin\RealtimeSync_Win32.exe"
  Delete "$INSTDIR\Bin\RealtimeSync_x64.exe"
  Delete "$INSTDIR\Bin\Shadow_Server2003_Win32.dll"
  Delete "$INSTDIR\Bin\Shadow_Server2003_x64.dll"
  Delete "$INSTDIR\Bin\Shadow_XP_Win32.dll"
  Delete "$INSTDIR\Bin\Shadow_XP_x64.dll" 
  Delete "$INSTDIR\Bin\FileOperation_Win32.dll"
  Delete "$INSTDIR\Bin\FileOperation_x64.dll"
  Delete "$INSTDIR\Bin\Taskbar7_Win32.dll"
  Delete "$INSTDIR\Bin\Taskbar7_x64.dll"
  Delete "$INSTDIR\Bin\Thumbnail_Win32.dll"  
  Delete "$INSTDIR\Bin\Thumbnail_x64.dll"  

  Delete "$INSTDIR\Languages\*.lng"

  ;remove config files: should only exist for portable version
  Delete "$INSTDIR\LastRun.ffs_gui"
  Delete "$INSTDIR\LastRun.ffs_real"
  Delete "$INSTDIR\GlobalSettings.xml"
  Delete "$INSTDIR\LastError.txt"
  Delete "$INSTDIR\Logs\*.log"

  ;leave the old stuff in (uninstall old versions!)
  Delete "$INSTDIR\Shadow_Server2003_Win32.dll"
  Delete "$INSTDIR\Shadow_Server2003_x64.dll"
  Delete "$INSTDIR\Shadow_XP_Win32.dll"
  Delete "$INSTDIR\Shadow_XP_x64.dll" 
  Delete "$INSTDIR\Recycler_Win32.dll"
  Delete "$INSTDIR\Recycler_x64.dll"
  Delete "$INSTDIR\FileOperation_Win32.dll"
  Delete "$INSTDIR\FileOperation_x64.dll"
  Delete "$INSTDIR\Taskbar7_Win32.dll"
  Delete "$INSTDIR\Taskbar7_x64.dll"
  Delete "$INSTDIR\mingwm10.dll"
  Delete "$INSTDIR\libgcc_s_dw2-1.dll"
  Delete "$INSTDIR\Shadow.zip"
  Delete "$INSTDIR\Shadow.dll"
  Delete "$INSTDIR\Readme.txt"
  
  ;remove (hopefully empty) directories
  RMDir "$INSTDIR\Bin"
  RMDir "$INSTDIR\Languages"
  RMDir "$INSTDIR\Logs"
  RMDir "$INSTDIR" ;remove if empty

  ;Remove shortcuts, if any
  SetShellVarContext all ;delete shortcuts for all users
  Delete "$SMPROGRAMS\FreeFileSync\*.*"
  RMDir "$SMPROGRAMS\FreeFileSync"
  Delete "$DESKTOP\FreeFileSync.lnk"
  SetShellVarContext current ;restore value

  ;print info about install progress
  DetailPrint $(Lang_Unregistering_FreeFileSync_file_extensions)

  ;/Remove registry keys:/
  ;entry in windows software catalog
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\FreeFileSync"
  ;*.ffs_gui association
  DeleteRegKey HKLM "Software\Classes\.ffs_gui"
  ;*.ffs_batch association
  DeleteRegKey HKLM "Software\Classes\.ffs_batch"
  ;*.ffs_db files
  DeleteRegKey HKLM "Software\Classes\.ffs_db"
  ;*.ffs_real association
  DeleteRegKey HKLM "Software\Classes\.ffs_real"
  ${un.RefreshShellIcons} 
SectionEnd


;################################################################################################################################################
;Installer Sections

!macro EnableSection SectionName
	SectionGetFlags ${SectionName} $0
	IntOp $0 $0 | ${SF_SELECTED}
	SectionSetFlags ${SectionName} $0	
!macroend


!macro DisbleSection SectionName
	SectionGetFlags ${SectionName} $0
	IntOp $1 ${SF_SELECTED} ~	
	IntOp $0 $0 & $1
	SectionSetFlags ${SectionName} $0	
!macroend


Function .onInit
;support for silent mode installer
IfSilent isSilent notSilent
	isSilent:
	;all sections are enabled by default...
	/*
	${If} ${RunningX64}
		!insertmacro DisbleSection ${Sec32BitFiles}
	${Else}
		!insertmacro DisbleSection ${Sec64BitFiles}
	${EndIf}
	*/
	notSilent:
FunctionEnd


Function nsDialogsPage
	;page heading
	!insertmacro MUI_HEADER_TEXT $(Lang_Prepare_installation) $(Lang_Choose_which_components_of_FreeFileSync_you_want_to_install)
 
	nsDialogs::Create 1018
	Pop $0

	;install type
	${NSD_CreateGroupBox} 0 0 100% 57u $(Lang_Select_installation_type)
	Pop $0

    ${NSD_CreateRadioButton} 10u 10u 120u 10u "$(Lang_Local) $(Lang_recommended)"
	Pop $InstallTypeLocal
	GetFunctionAddress $1 OnSettingsChange
	nsDialogs::OnClick $InstallTypeLocal $1  ;register callback when selection changes
	${NSD_AddStyle} $InstallTypeLocal ${WS_GROUP}|${WS_TABSTOP}	;begin new group of radiobuttons
		
	${NSD_CreateRadioButton} 130u 10u 120u 12u "$(Lang_Portable)"
	Pop $0
	GetFunctionAddress $1 OnSettingsChange
	nsDialogs::OnClick $0 $1  ;register callback when selection changes

	;preselect install type 'local'
	${NSD_Check} $InstallTypeLocal

	;horizontal separator
	${NSD_CreateHLine} 5% 25u 90% 1 Line
	Pop $0
	
	;installation type description
	${NSD_CreateLabel} 10% 28u 90% 25u ""
	Pop $InstallTypeDescription
	
	CreateFont $1 "$(^Font)" "$(^FontSize)" "600"  
	SendMessage $InstallTypeDescription ${WM_SETFONT} $1 0

	SetCtlColors $InstallTypeDescription 0x808080 transparent ;change description color to grey

/*
	;architecture
	${NSD_CreateGroupBox} 0 65u 100% 30u $(Lang_Program_architecture)
	Pop $0

	;determine radio button label
	${If} ${RunningX64}
		StrCpy $2 $(Lang_32_Bit)
	${Else}
		StrCpy $2 "$(Lang_32_Bit) $(Lang_recommended)"
	${EndIf}
    ${NSD_CreateRadioButton} 10u 75u 120u 12u $2
	Pop $Architecture32Bit

	${NSD_AddStyle} $Architecture32Bit ${WS_GROUP}|${WS_TABSTOP} ;begin new group of radiobuttons

	;determine radio button label
	${If} ${RunningX64}
		StrCpy $2 "$(Lang_64_Bit) $(Lang_recommended)"
	${Else}
		StrCpy $2 $(Lang_64_Bit)
	${EndIf}
	${NSD_CreateRadioButton} 130u 75u 120u 12u $2
	Pop $0
	
	;preselect architecture
	${If} ${RunningX64}
		${NSD_Check} $0
	${Else}
		${NSD_Check} $Architecture32Bit
	${EndIf}
*/
	;shortcuts
	${NSD_CreateLabel} 0 75u 75% 10u $(Lang_Create_shortcuts)
	Pop $0
	
	${NSD_CreateCheckBox} 10u 90u 100u 10u $(Lang_Desktop)
	Pop $CheckBoxDesktopLink

	${NSD_CreateCheckBox} 110u 90u 100u 10u $(Lang_Startmenu)
	Pop $CheckBoxStartmenuLink

	;installation directory
	${NSD_CreateDirRequest} 0 -30u 230u 12u ""
	Pop $SelectedDirectory
	
	;set default install directory
	${NSD_SetText} $SelectedDirectory "$INSTDIR"
	
	${NSD_CreateBrowseButton} 232u -31u 60u 14u $(Lang_Browse)
	Pop $0
	GetFunctionAddress $1 OnSelectFolder
	nsDialogs::OnClick $0 $1  ;register callback

	Push 0 ;Callback function expects something on stack
	Call OnSettingsChange
	
	nsDialogs::Show
FunctionEnd


Function OnSettingsChange
	Pop $0 ;nsDialogs callback function! HWND on stack!
	
	;enable/disable shortcuts depending on install type
	${NSD_GetState} $InstallTypeLocal $0
	${If} $0 == ${BST_CHECKED}
		${NSD_Check} $CheckBoxStartmenuLink
		${NSD_Check} $CheckBoxDesktopLink		
		;adapt description
		${NSD_SetText} $InstallTypeDescription "$(Lang_Register_FreeFileSync_file_extensions)$\n$(Lang_Create_uninstaller)$\n$(Lang_Save_settings_to_APPDATA_FreeFileSync)"
	
	${Else} ;portable
		${NSD_Uncheck} $CheckBoxStartmenuLink
		${NSD_Uncheck} $CheckBoxDesktopLink
		;adapt description
		${NSD_SetText} $InstallTypeDescription "$(Lang_Just_copy_the_files)$\n$(Lang_Do_not_write_to_Registry)$\n$(Lang_Save_settings_in_installation_directory)"
	${EndIf}
	
	;refresh background of description control
	ShowWindow $InstallTypeDescription ${SW_HIDE}
	ShowWindow $InstallTypeDescription ${SW_SHOW}
FunctionEnd


Function OnSelectFolder
	Pop $0 ;nsDialogs callback function! HWND on stack!
	
	${NSD_GetText} $SelectedDirectory $0
	nsDialogs::SelectFolderDialog $(Lang_Choose_a_directory_for_installation) $0
	Pop $0
	${If} $0 != error
		${NSD_SetText} $SelectedDirectory "$0" 
	${EndIf} 
FunctionEnd


Function nsDialogsPageLeave
	;evaluate selected options: first deactivate all sections
	!insertmacro DisbleSection ${SecInstallTypeInstaller} ;register filetypes and create uninstaller
	;!insertmacro DisbleSection ${Sec32BitFiles}
	;!insertmacro DisbleSection ${Sec64BitFiles}
	!insertmacro DisbleSection ${SecDesktopShortcut}
	!insertmacro DisbleSection ${SecStartmenuEntry}

	;update installation directory
	${NSD_GetText} $SelectedDirectory $0
	StrCpy $INSTDIR "$0"

	;installation type
	${NSD_GetState} $InstallTypeLocal $0
	${If} $0 == ${BST_CHECKED} ;local
		!insertmacro EnableSection ${SecInstallTypeInstaller} ;register filetypes and create uninstaller
	${Else} ;portable
	${EndIf}
	/*
	;architecture
	${NSD_GetState} $Architecture32Bit $0
	${If} $0 == ${BST_CHECKED} ;32-bit
		!insertmacro EnableSection ${Sec32BitFiles}
	${Else} ;64-bit
		!insertmacro EnableSection ${Sec64BitFiles}
	${EndIf}
	*/
	;link to desktop
	${NSD_GetState} $CheckBoxDesktopLink $0
	${If} $0 == ${BST_CHECKED}
		!insertmacro EnableSection ${SecDesktopShortcut}
	${EndIf}

	;links in startmenu
	${NSD_GetState} $CheckBoxStartmenuLink $0
	${If} $0 == ${BST_CHECKED}
		!insertmacro EnableSection ${SecStartmenuEntry}
	${EndIf}
FunctionEnd
