// **************************************************************************
// * This file is part of the zenXML project. It is distributed under the   *
// * Boost Software License, Version 1.0. See accompanying file             *
// * LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt.       *
// * Copyright (C) 2011 ZenJu (zhnmju123 AT gmx.de)                         *
// **************************************************************************

#ifndef ZEN_XML_IO_HEADER_8917640501480763248343343
#define ZEN_XML_IO_HEADER_8917640501480763248343343

#include <set>
#include <map>
#include <deque>
#include <vector>
#include <list>
#include <utility>
#include "zenxml.h"

class Dummy {};

void unit_test()
{
    //compile time checks

    using namespace zen;

    assert_static(!IsStlContainer<wchar_t>                     ::Result);
    assert_static(!IsStlContainer<char>                        ::Result);
    assert_static(!IsStlContainer<Dummy>                       ::Result);
    assert_static(!IsStlContainer<Loki::EmptyType>             ::Result);
    assert_static(IsStlContainer<std::set      <int>>          ::Result);
    assert_static(IsStlContainer<std::deque    <float>>        ::Result);
    assert_static(IsStlContainer<std::list     <size_t>>       ::Result);
    assert_static((IsStlContainer<std::map     <double, char>> ::Result));
    assert_static((IsStlContainer<std::multimap<short, double>>::Result));
    assert_static(IsStlContainer <std::vector  <wchar_t>>      ::Result);

    assert_static((IsStlPair     <std::pair<int, double>>      ::Result));
    assert_static(!IsStlPair<Dummy>                            ::Result);

    assert_static(!StringTraits<Dummy>::isStringLike);
    assert_static(!StringTraits<int>::isStringLike);
    assert_static(!StringTraits<double>::isStringLike);
    assert_static(!StringTraits<short>::isStringLike);
    assert_static(StringTraits<char>::isStringLike);
    assert_static(StringTraits<const wchar_t>::isStringLike);
    assert_static(StringTraits<const char>::isStringLike);
    assert_static(StringTraits<wchar_t>::isStringLike);
    assert_static(StringTraits<char*>::isStringLike);
    assert_static(StringTraits<wchar_t*>::isStringLike);
    assert_static(StringTraits<char * const>::isStringLike);
    assert_static(StringTraits<wchar_t * const>::isStringLike);
    assert_static(StringTraits<const char*>::isStringLike);
    assert_static(StringTraits<const char * const>::isStringLike);
    assert_static(StringTraits<const wchar_t*>::isStringLike);
    assert_static(StringTraits<const wchar_t * const>::isStringLike);
    assert_static(StringTraits<const char[4]>::isStringLike);
    assert_static(StringTraits<const wchar_t[4]>::isStringLike);
    assert_static(StringTraits<char[4]>::isStringLike);
    assert_static(StringTraits<wchar_t[4]>::isStringLike);
    assert_static(StringTraits<std::string>::isStringLike);
    assert_static(StringTraits<std::wstring>::isStringLike);
    assert_static(StringTraits<const std::string>::isStringLike);
    assert_static(StringTraits<const std::wstring>::isStringLike);
    assert_static(StringTraits<const std::string&>::isStringLike);
    assert_static(StringTraits<const std::wstring&>::isStringLike);
    assert_static(StringTraits<std::string&>::isStringLike);
    assert_static(StringTraits<std::wstring&>::isStringLike);

    assert_static(!(Loki::IsSameType<StringTraits<int>::CharType, char>::value));
    assert_static(!(Loki::IsSameType<StringTraits<double>::CharType, char>::value));
    assert_static(!(Loki::IsSameType<StringTraits<short>::CharType, char>::value));

    assert_static((Loki::IsSameType<StringTraits<char>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<wchar_t>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<const char>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<const wchar_t>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<char*>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<wchar_t*>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<char * const>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<wchar_t * const>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<const char*>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<const char * const>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<const wchar_t*>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<const wchar_t * const>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<const char[4]>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<const wchar_t[4]>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<char[4]>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<wchar_t[4]>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<std::string>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<std::wstring>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<const std::string>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<const std::wstring>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<const std::string&>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<const std::wstring&>::CharType, wchar_t>::value));
    assert_static((Loki::IsSameType<StringTraits<std::string&>::CharType, char>::value));
    assert_static((Loki::IsSameType<StringTraits<std::wstring&>::CharType, wchar_t>::value));
}

#endif //ZEN_XML_IO_HEADER_8917640501480763248343343
